#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
  REST_ARCH
  ~~~~~~~

  RESTful Framework Based On Gunicorn
"""

version_info = (0, 1, 1, 0)
__version__ = ".".join([str(v) for v in version_info])
